@@ -447,7 +447,7 @@ IMATH_INTERNAL_NAMESPACE_HEADER_ENTER
 
 ///
 ///
-/// class half -- 16-bit floating point number
+/// class half represents a 16-bit floating point number
 ///
 /// Type half can represent positive and negative numbers whose
 /// magnitude is between roughly 6.1e-5 and 6.5e+4 with a relative
@@ -620,10 +620,10 @@ class IMATH_EXPORT_TYPE half
     /// @name Access to the internal representation
 
     /// Return the bit pattern
-    IMATH_EXPORT constexpr uint16_t bits() const IMATH_NOEXCEPT;
+    constexpr uint16_t bits () const IMATH_NOEXCEPT;
 
     /// Set the bit pattern
-    IMATH_EXPORT IMATH_CONSTEXPR14 void setBits (uint16_t bits) IMATH_NOEXCEPT;
+    IMATH_CONSTEXPR14 void setBits (uint16_t bits) IMATH_NOEXCEPT;
 
     /// @}
 
@@ -979,7 +979,7 @@ IMATH_EXPORT void printBits (std::ostream& os, float f);
 IMATH_EXPORT void printBits (char c[19], IMATH_INTERNAL_NAMESPACE::half h);
 IMATH_EXPORT void printBits (char c[35], float f);
 
-#    ifndef __CUDACC__
+#    if !defined(__CUDACC__) && !defined(__CUDA_FP16_HPP__)
 using half = IMATH_INTERNAL_NAMESPACE::half;
 #    else
 #        include <cuda_fp16.h>
